/**
 * file    slimopt.c
 * brief   "slim" options parser
 * author  Yasuaki Nakanishi <yasuaki@tayaya.info>
 * source  http://sourceforge.jp/projects/slimopt
 *
 * MIT LICENCE
 * ===========
 * The licence text below is the boilerplate "MIT Licence" used from:
 * http: *www.opensource.org/licenses/mit-license.php
 *
 * Copyright (c) 2009, Yasuaki Nakanishi
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS 
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY 
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, 
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 */ 

/*----------------------------------------------------------------------------
 *  INCLUDE
 *----------------------------------------------------------------------------*/
#include "slimopt.h"

/*----------------------------------------------------------------------------
 *  DEFINE, TYPEDEF
 *----------------------------------------------------------------------------*/
#ifndef NULL
#define NULL  0
#endif

/*----------------------------------------------------------------------------
 *  FUNTIONS
 *----------------------------------------------------------------------------*/
/*----------------------------------------------------------------------------
 *  slimopt
 *----------------------------------------------------------------------------*/
/**
 *  parse the options on the command line
 *  param[in]  argc   : "argc" passed the function(int argc, char **argv)
 *  param[in]  argv   : "argv" passed the function(int argc, char **argv)
 *  param[in]  p_spec : OPTSPEC array
 *  param[out] p_val  : arry of void * values
 *  param[out] next_arg_id : next index of argv (0 means no next index)
 *  retval     err:
 *               0 OK
 *              -1 error:no argment
 *              -2 error:unmathed with the defined options
 *              -3 error:spec table may be illegal
 *
 *  note       The parsed value or default value will be filled in p_val array.
 */
int slimopt(int argc, char **argv, const OPTSPEC *p_spec, void **p_val, int *p_next_arg_id)
{
	int idx;
	int opt_id;
	int do_flg;
	int ng_flg;
	const char *p_tmp;

	*p_next_arg_id = 0;
	if ((p_spec == NULL) || (p_val == NULL)) return 0;

	/* initalization */
	idx = 0;
	while (p_spec[idx].short_name != '\0') {
		p_val[idx] = (void *)p_spec[idx].def_val;
		idx++;
	}

	if (argc <= 1) return 0;

	/* parse */
	for(idx = 1; idx < argc; idx++) {
		/* end of options with non option token? */
		if (argv[idx][0] != '-') {
			*p_next_arg_id = idx;
			return 0;
		}

		/* single "-" option? */
		if (argv[idx][1] == '\0') {
			*p_next_arg_id = idx;
			return 0;
		}

		/* end of options with "--"? */
		if ((argv[idx][1] == '-') && (argv[idx][2] == '\0')) {
			/* end of options "--" */
			idx++;
			if (idx >= argc) {
				idx = 0;
			}
			*p_next_arg_id = idx;
			return 0;
		}
		
		/* -abcd...? */
		p_tmp = &argv[idx][1];
		do_flg = 1;
		while ((do_flg == 1) && (*p_tmp != '\0')) {
			/* option? */
			ng_flg = 1;
			for(opt_id = 0; p_spec[opt_id].short_name != '\0'; opt_id++) {
				if (*p_tmp == p_spec[opt_id].short_name) {
					ng_flg = 0;
					break;
				}
			} 

			if (ng_flg) {
				*p_next_arg_id = idx;
				return -2; /* unmathed with the defined options */
			}

			switch (p_spec[opt_id].type) {
			case OPT_FLG:
				p_val[opt_id] = (void *)1;
				p_tmp++;
				break;
			case OPT_STR:
				idx++;
				p_tmp++;
				if ( '\0' != *p_tmp) {
					idx--;
					p_val[opt_id] = (void *)p_tmp;
				} else if (idx >= argc) {
					*p_next_arg_id = idx - 1;
					return -1; /* no argment */
				} else {
					p_val[opt_id] = (void *)argv[idx];
				}
				do_flg = 0;
				break;
			default:
				*p_next_arg_id = idx;
				return -3; /* type error : may be used illegal spec table */
				break;
			}
		}
	}

	*p_next_arg_id = 0;
	return 0;
} 

/* end of slimopt.c */
